from typing import Any


def get_hello_data(client_key: str | None = None) -> dict[str, Any]:
    return {
        "id": "register_0",
        "payload": {
            "client-key": client_key,
            "forcePairing": False,
            "manifest": {
                "appVersion": "1.1",
                "manifestVersion": 1,
                "permissions": [
                    "ADD_LAUNCHER_CHANNEL",
                    "APP_TO_APP",
                    "CHECK_BLUETOOTH_DEVICE",
                    "CLOSE",
                    "CONTROL_AUDIO",
                    "CONTROL_BLUETOOTH",
                    "CONTROL_BOX_CHANNEL",
                    "CONTROL_CHANNEL_BLOCK",
                    "CONTROL_CHANNEL_GROUP",
                    "CONTROL_DISPLAY",
                    "CONTROL_FAVORITE_GROUP",
                    "CONTROL_INPUT_JOYSTICK",
                    "CONTROL_INPUT_MEDIA_PLAYBACK",
                    "CONTROL_INPUT_MEDIA_RECORDING",
                    "CONTROL_INPUT_TV",
                    "CONTROL_POWER",
                    "CONTROL_RECORDING",
                    "CONTROL_TIMER_INFO",
                    "CONTROL_TV_POWER",
                    "CONTROL_TV_SCREEN",
                    "CONTROL_TV_STANBY",
                    "CONTROL_USER_INFO",
                    "CONTROL_WOL",
                    "DELETE_SELECT_CHANNEL",
                    "LAUNCH",
                    "LAUNCH_WEBAPP",
                    "READ_APP_STATUS",
                    "READ_COUNTRY_INFO",
                    "READ_CURRENT_CHANNEL",
                    "READ_INPUT_DEVICE_LIST",
                    "READ_NETWORK_STATE",
                    "READ_POWER_STATE",
                    "READ_RECORDING_LIST",
                    "READ_RECORDING_SCHEDULE",
                    "READ_RECORDING_STATE",
                    "READ_RUNNING_APPS",
                    "READ_SETTINGS",
                    "READ_STORAGE_DEVICE_LIST",
                    "READ_TV_ACR_AUTH_TOKEN",
                    "READ_TV_CHANNEL_LIST",
                    "READ_TV_CONTENT_STATE",
                    "READ_TV_CURRENT_TIME",
                    "READ_TV_PROGRAM_INFO",
                    "RELEASE_CHANNEL_SKIP",
                    "SCAN_TV_CHANNELS",
                    "SET_CHANNEL_SKIP",
                    "STB_INTERNAL_CONNECTION",
                    "TEST_OPEN",
                    "TEST_PROTECTED",
                    "WRITE_NOTIFICATION_TOAST",
                    "WRITE_RECORDING_LIST",
                    "WRITE_RECORDING_SCHEDULE",
                ],
                "signatures": [
                    {
                        "signature": "eyJhbGdvcml0aG0iOiJSU0EtU0hBMjU2Iiwia2V5SWQiOiJ0ZXN0LXNpZ25pbmctY2VydCIsInNpZ25hdHVyZVZlcnNpb24iOjF9.hrVRgjCwXVvE2OOSpDZ58hR+59aFNwYDyjQgKk3auukd7pcegmE2CzPCa0bJ0ZsRAcKkCTJrWo5iDzNhMBWRyaMOv5zWSrthlf7G128qvIlpMT0YNY+n/FaOHE73uLrS/g7swl3/qH/BGFG2Hu4RlL48eb3lLKqTt2xKHdCs6Cd4RMfJPYnzgvI4BNrFUKsjkcu+WD4OO2A27Pq1n50cMchmcaXadJhGrOqH5YmHdOCj5NSHzJYrsW0HPlpuAx/ECMeIZYDh6RMqaFM2DXzdKX9NmmyqzJ3o/0lkk/N97gfVRLW5hA29yeAwaCViZNCP8iC9aO0q9fQojoa7NQnAtw==",
                        "signatureVersion": 1,
                    }
                ],
                # These can't be changed, then we get "insufficient permissions" errors
                "signed": {
                    "appId": "com.lge.test",
                    "created": "20140509",
                    "localizedAppNames": {
                        "": "LG Remote App",
                        "ko-KR": "리모컨 앱",
                        "zxx-XX": "ЛГ Rэмotэ AПП",
                    },
                    "localizedVendorNames": {"": "LG Electronics"},
                    "permissions": [
                        "TEST_SECURE",
                        "CONTROL_INPUT_TEXT",
                        "CONTROL_MOUSE_AND_KEYBOARD",
                        "READ_INSTALLED_APPS",
                        "READ_LGE_SDX",
                        "READ_NOTIFICATIONS",
                        "SEARCH",
                        "WRITE_SETTINGS",
                        "WRITE_NOTIFICATION_ALERT",
                        "CONTROL_POWER",
                        "READ_CURRENT_CHANNEL",
                        "READ_RUNNING_APPS",
                        "READ_UPDATE_INFO",
                        "UPDATE_FROM_REMOTE_APP",
                        "READ_LGE_TV_INPUT_EVENTS",
                        "READ_TV_CURRENT_TIME",
                    ],
                    "serial": "2f930e2d2cfe083771f68e4fe7bb07",
                    "vendorId": "com.lge",
                },
            },
            "pairingType": "PROMPT",
        },
        "type": "register",
    }



from typer import Typer
from wakeonlan import send_magic_packet

from alga import client


app = Typer(no_args_is_help=True)


@app.command()
def off() -> None:
    client.request("ssap://system/turnOff")


@app.command()
def on(mac: str) -> None:
    send_magic_packet(mac)



import json
from ipaddress import ip_address
from socket import gaierror, getaddrinfo
from typing import Annotated

from rich import print
from typer import Argument, Exit

from alga import client
from alga.payloads import get_hello_data


def _ip_from_hostname(hostname: str) -> str | None:
    try:
        results = getaddrinfo(host=hostname, port=None)
        # TODO: Do we want to handle receiving multiple IP addresses?
        return results[0][4][0]
    except gaierror:
        return None


def setup(hostname: Annotated[str, Argument()] = "lgwebostv") -> None:
    # Check if we have been passed an IP address
    ip: str | None
    try:
        ip = ip_address(hostname).compressed
    except ValueError:
        ip = _ip_from_hostname(hostname)
        if not ip:
            print(
                f"[red]Could not find any host by the name '{hostname}'.[/red] Is the TV on and connected to the network?"
            )
            raise Exit(code=1)

    with client.new(
        perform_handshake=False, timeout=60
    ) as connection:  # pragma: no cover
        connection.send(json.dumps(get_hello_data()))
        response = json.loads(connection.recv())
        assert response == {
            "id": "register_0",
            "payload": {"pairingType": "PROMPT", "returnValue": True},
            "type": "response",
        }, "Unexpected response received"
        print("Please approve the connection request on the TV now...")

        response = json.loads(connection.recv())
        if "client-key" not in response["payload"]:
            print("[red]Setup failed![/red]")
            raise Exit(code=1)

        print(
            f"Got key: {response['payload']['client-key']}. Please put this in the `ALGA_KEY` environment variable"
        )



from typing import Annotated

from rich.console import Console
from rich.table import Table
from typer import Argument, Typer

from alga import client


app = Typer(no_args_is_help=True)


@app.command()
def set(value: Annotated[str, Argument()]) -> None:
    client.request("ssap://tv/switchInput", {"inputId": value})


@app.command()
def list() -> None:
    response = client.request("ssap://tv/getExternalInputList")

    table = Table()
    table.add_column("Name")
    table.add_column("ID")

    all_inputs = []
    for i in response["devices"]:
        all_inputs.append([i["label"], i["id"]])

    for row in all_inputs:
        table.add_row(*row)

    console = Console()
    console.print(table)



from typer import Typer

from alga import client


app = Typer(no_args_is_help=True)


@app.command()
def play() -> None:
    client.request("ssap://media.controls/play")


@app.command()
def pause() -> None:
    client.request("ssap://media.controls/pause")


@app.command()
def stop() -> None:
    client.request("ssap://media.controls/stop")


@app.command()
def fast_forward() -> None:
    client.request("ssap://media.controls/fastForward")


@app.command()
def rewind() -> None:
    client.request("ssap://media.controls/rewind")



import json
from typing import Annotated, Optional

from rich import print
from typer import Argument

from alga import client


def adhoc(path: str, data: Annotated[Optional[str], Argument()] = None) -> None:
    if data:
        print(client.request(path, json.loads(data)))
    else:
        print(client.request(path))



from typing import Annotated

from rich import print
from typer import Argument, Typer

from alga import client


app = Typer(no_args_is_help=True)


@app.command()
def up() -> None:
    client.request("ssap://audio/volumeUp")


@app.command()
def down() -> None:
    client.request("ssap://audio/volumeDown")


@app.command()
def set(value: Annotated[int, Argument()]) -> None:
    client.request("ssap://audio/setVolume", {"volume": value})


@app.command()
def get() -> None:
    response = client.request("ssap://audio/getVolume")
    print(
        f"Volume is currently set to [bold]{response['volume']}[/bold] and is currently {'[red]' if response['muted'] else '[green]not '}muted"
    )


@app.command()
def mute() -> None:
    client.request("ssap://audio/setMute", {"mute": True})


@app.command()
def unmute() -> None:
    client.request("ssap://audio/setMute", {"mute": False})



from typing import Annotated

from rich import print
from rich.console import Console
from rich.table import Table
from typer import Argument, Typer

from alga import client


app = Typer(no_args_is_help=True)


@app.command()
def current() -> None:
    response = client.request("ssap://tv/getCurrentChannel")
    print(
        f"The current channel is [bold]{response['channelName']}[/bold] ([italic]{response['channelNumber']}[/italic])"
    )


@app.command()
def up() -> None:
    client.request("ssap://tv/channelUp")


@app.command()
def down() -> None:
    client.request("ssap://tv/channelDown")


@app.command()
def set(value: Annotated[int, Argument()]) -> None:
    client.request("ssap://tv/openChannel", {"channelNumber": value})


@app.command()
def list() -> None:
    response = client.request("ssap://tv/getChannelList")

    table = Table()
    table.add_column("Type")
    table.add_column("Number")
    table.add_column("Name")

    all_channels = []
    type_to_emoji = {1: "📺", 2: "📻"}
    for channel in response["channelList"]:
        # The first item is for sorting
        all_channels.append(
            [
                int(channel["channelNumber"]),
                type_to_emoji.get(channel["channelTypeId"], "❓"),
                channel["channelNumber"],
                channel["channelName"],
            ]
        )

    for row in sorted(all_channels):
        table.add_row(*row[1:])

    console = Console()
    console.print(table)



import json
import ssl
from collections.abc import Iterator
from contextlib import contextmanager
from os import getenv
from typing import Any, cast

from websocket import WebSocket

from alga.payloads import get_hello_data


HOSTNAME = getenv("ALGA_HOST", "lgwebostv")
KEY = getenv("ALGA_KEY", "")


@contextmanager
def new(
    perform_handshake: bool = True, timeout: int = 3
) -> Iterator[WebSocket]:  # pragma: no cover
    connection = WebSocket(sslopt={"cert_reqs": ssl.CERT_NONE})
    connection.connect(f"wss://{HOSTNAME}:3001/", suppress_origin=True, timeout=timeout)  # type: ignore[no-untyped-call]

    if perform_handshake:
        connection.send(json.dumps(get_hello_data(KEY)))
        response = json.loads(connection.recv())
        if "client-key" not in response["payload"]:
            raise Exception(
                f"Something went wrong with performing a handshake. Response: {response}"
            )

    try:
        yield connection
    finally:
        connection.close()


def request(
    uri: str, data: dict[str, Any] | None = None
) -> dict[str, Any]:  # pragma: no cover
    with new() as connection:
        request: dict[str, Any] = {"type": "request", "uri": uri}

        if data:
            request.update(payload=data)

        connection.send(json.dumps(request))

        raw_response = connection.recv()
        response = json.loads(raw_response)

        assert response.get("payload", {}).get("returnValue") is True
        return cast(dict[str, Any], response["payload"])
